@@ -27,6 +27,8 @@
 #include <range/v3/iterator/traits.hpp>
 #include <range/v3/utility/static_const.hpp>
 
+#include <range/v3/detail/prologue.hpp>
+
 namespace ranges
 {
     /// \addtogroup group-actions
@@ -35,16 +37,17 @@ namespace ranges
     {
         template<typename Rng>
         using join_action_value_t_ =
-            meta::if_c<(bool)ranges::container<range_value_t<Rng>>, range_value_t<Rng>,
+            meta::if_c<(bool)ranges::container<range_value_t<Rng>>, //
+                       range_value_t<Rng>,                          //
                        std::vector<range_value_t<range_value_t<Rng>>>>;
 
         struct join_fn
         {
-        public:
-            template<typename Rng>
-            auto operator()(Rng && rng) const -> CPP_ret(join_action_value_t_<Rng>)( //
-                requires input_range<Rng> && input_range<range_value_t<Rng>> &&
+            template(typename Rng)(
+                /// \pre
+                requires input_range<Rng> AND input_range<range_value_t<Rng>> AND
                     semiregular<join_action_value_t_<Rng>>)
+            join_action_value_t_<Rng> operator()(Rng && rng) const
             {
                 join_action_value_t_<Rng> ret;
                 auto last = ranges::end(rng);
@@ -54,12 +57,13 @@ namespace ranges
             }
         };
 
-        /// \ingroup group-actions
-        /// \relates join_fn
-        /// \sa action
-        RANGES_INLINE_VARIABLE(action<join_fn>, join)
+        /// \relates actions::join_fn
+        /// \sa action_closure
+        RANGES_INLINE_VARIABLE(action_closure<join_fn>, join)
     } // namespace actions
     /// @}
 } // namespace ranges
 
+#include <range/v3/detail/epilogue.hpp>
+
 #endif
