@@ -51,6 +51,8 @@
 #include <range/v3/utility/static_const.hpp>
 #include <range/v3/utility/swap.hpp>
 
+#include <range/v3/detail/prologue.hpp>
+
 namespace ranges
 {
     /// \cond
@@ -94,12 +96,12 @@ namespace ranges
             }
 
         public:
-            template<typename I, typename C = less, typename P = identity>
-            auto operator()(I first, I middle, I last, iter_difference_t<I> len1,
+            template(typename I, typename C = less, typename P = identity)(
+                /// \pre
+                requires bidirectional_iterator<I> AND sortable<I, C, P>)
+            void operator()(I first, I middle, I last, iter_difference_t<I> len1,
                             iter_difference_t<I> len2, iter_value_t<I> * buf,
                             std::ptrdiff_t buf_size, C pred = C{}, P proj = P{}) const
-                -> CPP_ret(void)( //
-                    requires bidirectional_iterator<I> && sortable<I, C, P>)
             {
                 using D = iter_difference_t<I>;
                 while(true)
@@ -219,11 +221,11 @@ namespace ranges
 
         struct inplace_merge_no_buffer_fn
         {
-            template<typename I, typename C = less, typename P = identity>
-            auto operator()(I first, I middle, I last, iter_difference_t<I> len1,
+            template(typename I, typename C = less, typename P = identity)(
+                /// \pre
+                requires bidirectional_iterator<I> AND sortable<I, C, P>)
+            void operator()(I first, I middle, I last, iter_difference_t<I> len1,
                             iter_difference_t<I> len2, C pred = C{}, P proj = P{}) const
-                -> CPP_ret(void)( //
-                    requires bidirectional_iterator<I> && sortable<I, C, P>)
             {
                 merge_adaptive(std::move(first),
                                std::move(middle),
@@ -243,16 +245,16 @@ namespace ranges
 
     /// \addtogroup group-algorithms
     /// @{
-    RANGES_BEGIN_NIEBLOID(inplace_merge)
+    RANGES_FUNC_BEGIN(inplace_merge)
 
         // TODO reimplement to only need forward iterators
 
         /// \brief function template \c inplace_merge
-        template<typename I, typename S, typename C = less, typename P = identity>
-        auto RANGES_FUN_NIEBLOID(inplace_merge)(
+        template(typename I, typename S, typename C = less, typename P = identity)(
+            /// \pre
+            requires bidirectional_iterator<I> AND sortable<I, C, P>)
+        I RANGES_FUNC(inplace_merge)(
             I first, I middle, S last, C pred = C{}, P proj = P{})
-            ->CPP_ret(I)( //
-                requires bidirectional_iterator<I> && sortable<I, C, P>)
         {
             using value_type = iter_value_t<I>;
             auto len1 = distance(first, middle);
@@ -260,7 +262,7 @@ namespace ranges
             auto buf_size = ranges::min(len1, len2_and_end.first);
             std::pair<value_type *, std::ptrdiff_t> buf{nullptr, 0};
             std::unique_ptr<value_type, detail::return_temporary_buffer> h;
-            if(detail::is_trivially_copy_assignable<value_type>::value && 8 < buf_size)
+            if(detail::is_trivially_copy_assignable_v<value_type> && 8 < buf_size)
             {
                 buf = detail::get_temporary_buffer<value_type>(buf_size);
                 h.reset(buf.first);
@@ -278,11 +280,11 @@ namespace ranges
         }
 
         /// \overload
-        template<typename Rng, typename C = less, typename P = identity>
-        auto RANGES_FUN_NIEBLOID(inplace_merge)(
+        template(typename Rng, typename C = less, typename P = identity)(
+            /// \pre
+            requires bidirectional_range<Rng> AND sortable<iterator_t<Rng>, C, P>)
+        borrowed_iterator_t<Rng> RANGES_FUNC(inplace_merge)(
             Rng && rng, iterator_t<Rng> middle, C pred = C{}, P proj = P{})
-            ->CPP_ret(safe_iterator_t<Rng>)( //
-                requires bidirectional_range<Rng> && sortable<iterator_t<Rng>, C, P>)
         {
             return (*this)(begin(rng),
                            std::move(middle),
@@ -291,7 +293,7 @@ namespace ranges
                            std::move(proj));
         }
 
-    RANGES_END_NIEBLOID(inplace_merge)
+    RANGES_FUNC_END(inplace_merge)
 
     namespace cpp20
     {
@@ -300,4 +302,6 @@ namespace ranges
     /// @}
 } // namespace ranges
 
-#endif // include guard
+#include <range/v3/detail/epilogue.hpp>
+
+#endif
