@@ -31,7 +31,7 @@
 #include <range/v3/range/traits.hpp>
 #include <range/v3/utility/static_const.hpp>
 
-#include <range/v3/detail/disable_warnings.hpp>
+#include <range/v3/detail/prologue.hpp>
 
 namespace ranges
 {
@@ -43,19 +43,19 @@ namespace ranges
     RANGES_FUNC_BEGIN(partition_copy)
 
         /// \brief function template \c partition_copy
-        template<typename I,
+        template(typename I,
                  typename S,
                  typename O0,
                  typename O1,
                  typename C,
-                 typename P = identity>
-        auto RANGES_FUNC(partition_copy)(
-            I first, S last, O0 o0, O1 o1, C pred, P proj = P{})
-            ->CPP_ret(partition_copy_result<I, O0, O1>)( //
-                requires input_iterator<I> && sentinel_for<S, I> &&
-                weakly_incrementable<O0> && weakly_incrementable<O1> &&
-                indirectly_copyable<I, O0> && indirectly_copyable<I, O1> &&
+                 typename P = identity)(
+            /// \pre
+            requires input_iterator<I> AND sentinel_for<S, I> AND
+                weakly_incrementable<O0> AND weakly_incrementable<O1> AND
+                indirectly_copyable<I, O0> AND indirectly_copyable<I, O1> AND
                 indirect_unary_predicate<C, projected<I, P>>)
+        partition_copy_result<I, O0, O1> RANGES_FUNC(partition_copy)(
+            I first, S last, O0 o0, O1 o1, C pred, P proj = P{})
         {
             for(; first != last; ++first)
             {
@@ -75,18 +75,18 @@ namespace ranges
         }
 
         /// \overload
-        template<typename Rng,
+        template(typename Rng,
                  typename O0,
                  typename O1,
                  typename C,
-                 typename P = identity>
-        auto RANGES_FUNC(partition_copy)(
-            Rng && rng, O0 o0, O1 o1, C pred, P proj = P{})                 //
-            ->CPP_ret(partition_copy_result<safe_iterator_t<Rng>, O0, O1>)( //
-                requires input_range<Rng> && weakly_incrementable<O0> &&
-                weakly_incrementable<O1> && indirectly_copyable<iterator_t<Rng>, O0> &&
-                indirectly_copyable<iterator_t<Rng>, O1> &&
+                 typename P = identity)(
+            /// \pre
+            requires input_range<Rng> AND weakly_incrementable<O0> AND
+                weakly_incrementable<O1> AND indirectly_copyable<iterator_t<Rng>, O0> AND
+                indirectly_copyable<iterator_t<Rng>, O1> AND
                 indirect_unary_predicate<C, projected<iterator_t<Rng>, P>>)
+        partition_copy_result<borrowed_iterator_t<Rng>, O0, O1> //
+        RANGES_FUNC(partition_copy)(Rng && rng, O0 o0, O1 o1, C pred, P proj = P{})
         {
             return (*this)(begin(rng),
                            end(rng),
@@ -106,6 +106,6 @@ namespace ranges
     /// @}
 } // namespace ranges
 
-#include <range/v3/detail/reenable_warnings.hpp>
+#include <range/v3/detail/epilogue.hpp>
 
 #endif
