@@ -46,6 +46,8 @@
 #include <range/v3/utility/static_const.hpp>
 #include <range/v3/utility/swap.hpp>
 
+#include <range/v3/detail/prologue.hpp>
+
 namespace ranges
 {
     /// \addtogroup group-algorithms
@@ -56,7 +58,7 @@ namespace ranges
     {
         template<typename I, typename C, typename P, typename D, typename Pair>
         I stable_partition_impl(I first, I last, C pred, P proj, D len, Pair const p,
-                                detail::forward_iterator_tag_ fi)
+                                std::forward_iterator_tag fi)
         {
             // *first is known to be false
             // len >= 1
@@ -128,7 +130,7 @@ namespace ranges
 
         template<typename I, typename S, typename C, typename P>
         I stable_partition_impl(I first, S last, C pred, P proj,
-                                detail::forward_iterator_tag_ fi)
+                                std::forward_iterator_tag fi)
         {
             using difference_type = iter_difference_t<I>;
             difference_type const alloc_limit = 3; // might want to make this a function
@@ -156,7 +158,7 @@ namespace ranges
 
         template<typename I, typename C, typename P, typename D, typename Pair>
         I stable_partition_impl(I first, I last, C pred, P proj, D len, Pair p,
-                                detail::bidirectional_iterator_tag_ bi)
+                                std::bidirectional_iterator_tag bi)
         {
             // *first is known to be false
             // *last is known to be true
@@ -251,7 +253,7 @@ namespace ranges
 
         template<typename I, typename S, typename C, typename P>
         I stable_partition_impl(I first, S end_, C pred, P proj,
-                                detail::bidirectional_iterator_tag_ bi)
+                                std::bidirectional_iterator_tag bi)
         {
             using difference_type = iter_difference_t<I>;
             using value_type = iter_value_t<I>;
@@ -286,17 +288,16 @@ namespace ranges
             return detail::stable_partition_impl(first, last, pred, proj, len, p, bi);
         }
     } // namespace detail
-    /// endcond
+    /// \endcond
 
-    RANGES_BEGIN_NIEBLOID(stable_partition)
+    RANGES_FUNC_BEGIN(stable_partition)
 
         /// \brief function template \c stable_partition
-        template<typename I, typename S, typename C, typename P = identity>
-        auto RANGES_FUN_NIEBLOID(stable_partition)(
-            I first, S last, C pred, P proj = P{}) //
-            ->CPP_ret(I)(                          //
-                requires bidirectional_iterator<I> && sentinel_for<S, I> &&
-                indirect_unary_predicate<C, projected<I, P>> && permutable<I>)
+        template(typename I, typename S, typename C, typename P = identity)(
+            /// \pre
+            requires bidirectional_iterator<I> AND sentinel_for<S, I> AND
+            indirect_unary_predicate<C, projected<I, P>> AND permutable<I>)
+        I RANGES_FUNC(stable_partition)(I first, S last, C pred, P proj = P{})
         {
             return detail::stable_partition_impl(std::move(first),
                                                  std::move(last),
@@ -307,17 +308,18 @@ namespace ranges
 
         // BUGBUG Can this be optimized if Rng has O1 size?
         /// \overload
-        template<typename Rng, typename C, typename P = identity>
-        auto RANGES_FUN_NIEBLOID(stable_partition)(Rng && rng, C pred, P proj = P{}) //
-            ->CPP_ret(safe_iterator_t<Rng>)(                                         //
-                requires bidirectional_range<Rng> &&
-                indirect_unary_predicate<C, projected<iterator_t<Rng>, P>> &&
-                permutable<iterator_t<Rng>>)
+        template(typename Rng, typename C, typename P = identity)(
+            /// \pre
+            requires bidirectional_range<Rng> AND
+            indirect_unary_predicate<C, projected<iterator_t<Rng>, P>> AND
+            permutable<iterator_t<Rng>>)
+        borrowed_iterator_t<Rng> //
+        RANGES_FUNC(stable_partition)(Rng && rng, C pred, P proj = P{}) //
         {
             return (*this)(begin(rng), end(rng), std::move(pred), std::move(proj));
         }
 
-    RANGES_END_NIEBLOID(stable_partition)
+    RANGES_FUNC_END(stable_partition)
 
     namespace cpp20
     {
@@ -326,4 +328,6 @@ namespace ranges
     /// @}
 } // namespace ranges
 
-#endif // include guard
+#include <range/v3/detail/epilogue.hpp>
+
+#endif
