@@ -14,6 +14,8 @@
 #ifndef RANGES_V3_RANGE_CONVERSION_HPP
 #define RANGES_V3_RANGE_CONVERSION_HPP
 
+#include <vector>
+
 #include <meta/meta.hpp>
 
 #include <range/v3/range_fwd.hpp>
@@ -25,17 +27,7 @@
 #include <range/v3/range/traits.hpp>
 #include <range/v3/utility/static_const.hpp>
 
-#ifndef RANGES_NO_STD_FORWARD_DECLARATIONS
-// Non-portable forward declarations of standard containers
-RANGES_BEGIN_NAMESPACE_STD
-    RANGES_BEGIN_NAMESPACE_CONTAINER
-        template<typename Value, typename Alloc /*= allocator<Value>*/>
-        class vector;
-    RANGES_END_NAMESPACE_CONTAINER
-RANGES_END_NAMESPACE_STD
-#else
-#include <vector>
-#endif
+#include <range/v3/detail/prologue.hpp>
 
 namespace ranges
 {
@@ -44,18 +36,35 @@ namespace ranges
     {
         struct to_container
         {
-            template<typename ToContainer>
+            template<typename MetaFn>
             struct fn;
 
-            template<typename ToContainer, typename Rng>
-            using container_t = meta::invoke<ToContainer, Rng>;
+            template<typename MetaFn, typename Fn>
+            struct closure;
 
-            template<typename Rng, typename ToContainer>
-            friend auto operator|(Rng && rng, fn<ToContainer> (*)(to_container))
-                -> CPP_broken_friend_ret(container_t<ToContainer, Rng>)( //
-                    requires invocable<fn<ToContainer>, Rng>)
+            template<typename MetaFn, typename Rng>
+            using container_t = meta::invoke<MetaFn, Rng>;
+
+            template<typename Rng, typename MetaFn>
+            friend auto operator|(Rng && rng,
+                                  closure<MetaFn, fn<MetaFn>> (*)(to_container))
+                -> CPP_broken_friend_ret(container_t<MetaFn, Rng>)(
+                    /// \pre
+                    requires invocable<fn<MetaFn>, Rng>)
             {
-                return fn<ToContainer>{}(static_cast<Rng &&>(rng));
+                return fn<MetaFn>{}(static_cast<Rng &&>(rng));
+            }
+
+            template<typename MetaFn, typename Pipeable>
+            friend auto operator|(closure<MetaFn, fn<MetaFn>> (*)(to_container),
+                                  Pipeable pipe)
+                -> CPP_broken_friend_ret(
+                    closure<MetaFn, composed<Pipeable, fn<MetaFn>>>)(
+                    /// \pre
+                    requires (is_pipeable_v<Pipeable>))
+            {
+                return closure<MetaFn, composed<Pipeable, fn<MetaFn>>>{
+                    compose(static_cast<Pipeable &&>(pipe), fn<MetaFn>{})};
             }
         };
 
@@ -108,30 +117,42 @@ namespace ranges
                 return tmp;
             }
             CPP_member
-            auto operator--() -> CPP_ret(to_container_iterator &)(
-                requires derived_from<iterator_category, std::bidirectional_iterator_tag>)
+            auto operator--() //
+                -> CPP_ret(to_container_iterator &)(
+                    /// \pre
+                    requires derived_from<iterator_category,
+                                          std::bidirectional_iterator_tag>)
             {
                 --it_;
                 return *this;
             }
             CPP_member
-            auto operator--(int) -> CPP_ret(to_container_iterator &)(
-                requires derived_from<iterator_category, std::bidirectional_iterator_tag>)
+            auto operator--(int) //
+                -> CPP_ret(to_container_iterator &)(
+                    /// \pre
+                    requires derived_from<iterator_category,
+                                          std::bidirectional_iterator_tag>)
             {
                 auto tmp = *this;
                 ++it_;
                 return tmp;
             }
             CPP_member
-            auto operator+=(difference_type n) -> CPP_ret(to_container_iterator &)(
-                requires derived_from<iterator_category, std::random_access_iterator_tag>)
+            auto operator+=(difference_type n) //
+                -> CPP_ret(to_container_iterator &)(
+                    /// \pre
+                    requires derived_from<iterator_category,
+                                          std::random_access_iterator_tag>)
             {
                 it_ += n;
                 return *this;
             }
             CPP_member
-            auto operator-=(difference_type n) -> CPP_ret(to_container_iterator &)(
-                requires derived_from<iterator_category, std::random_access_iterator_tag>)
+            auto operator-=(difference_type n) //
+                -> CPP_ret(to_container_iterator &)(
+                    /// \pre
+                    requires derived_from<iterator_category,
+                                          std::random_access_iterator_tag>)
             {
                 it_ -= n;
                 return *this;
@@ -139,6 +160,7 @@ namespace ranges
             CPP_broken_friend_member
             friend auto operator+(to_container_iterator i, difference_type n) //
                 -> CPP_broken_friend_ret(to_container_iterator)(
+                    /// \pre
                     requires derived_from<iterator_category,
                                           std::random_access_iterator_tag>)
             {
@@ -147,6 +169,7 @@ namespace ranges
             CPP_broken_friend_member
             friend auto operator-(to_container_iterator i, difference_type n) //
                 -> CPP_broken_friend_ret(to_container_iterator)(
+                    /// \pre
                     requires derived_from<iterator_category,
                                           std::random_access_iterator_tag>)
             {
@@ -155,6 +178,7 @@ namespace ranges
             CPP_broken_friend_member
             friend auto operator-(difference_type n, to_container_iterator i) //
                 -> CPP_broken_friend_ret(to_container_iterator)(
+                    /// \pre
                     requires derived_from<iterator_category,
                                           std::random_access_iterator_tag>)
             {
@@ -164,14 +188,18 @@ namespace ranges
             friend auto operator-(to_container_iterator const & i,
                                   to_container_iterator const & j) //
                 -> CPP_broken_friend_ret(difference_type)(
+                    /// \pre
                     requires derived_from<iterator_category,
                                           std::random_access_iterator_tag>)
             {
                 return i.it_ - j.it_;
             }
             CPP_member
-            auto operator[](difference_type n) const -> CPP_ret(reference)(
-                requires derived_from<iterator_category, std::random_access_iterator_tag>)
+            auto operator[](difference_type n) const //
+                -> CPP_ret(reference)(
+                    /// \pre
+                    requires derived_from<iterator_category,
+                                          std::random_access_iterator_tag>)
             {
                 return *(*this + n);
             }
@@ -182,70 +210,107 @@ namespace ranges
             enable_if_t<(bool)range<Rng>, to_container_iterator<Rng, Cont>>;
 
         // clang-format off
-        CPP_def
-        (
-            template(typename Cont)
-            concept has_allocator_type,
-                ranges::type<typename Cont::allocator_type>
-        );
+        template(typename Rng)(
+        concept (range_and_not_view_)(Rng),
+            range<Rng> AND (!view_<Rng>));
 
-        CPP_def
-        (
-            template(typename Rng, typename Cont)
-            concept convertible_to_container_impl_,
-                range<Cont> && (!view_<Cont>) && move_constructible<Cont> &&
-                constructible_from<range_value_t<Cont>, range_reference_t<Rng>> &&
-                constructible_from<
-                    Cont,
-                    range_cpp17_iterator_t<Rng>,
-                    range_cpp17_iterator_t<Rng>>
-        );
+        template<typename Rng>
+        CPP_concept range_and_not_view =
+            CPP_concept_ref(range_and_not_view_, Rng);
 
-        CPP_def
-        (
-            template(typename Rng, typename Cont)
-            concept convertible_to_container_container_impl_,
-                range<Cont> && (!view_<Cont>) && move_constructible<Cont> &&
-                (bool(ranges::defer::range<range_value_t<Cont>> &&
-                    !ranges::defer::view_<range_value_t<Cont>>)) &&
-                // Test that each element of the input range can be ranges::to<>
-                // to the output container.
-                invocable<
-                    to_container::fn<meta::id<range_value_t<Cont>>>,
-                    range_reference_t<Rng>> &&
-                constructible_from<
-                    Cont,
-                    to_container_iterator_t<Rng, Cont>,
-                    to_container_iterator_t<Rng, Cont>>
+        template(typename Rng, typename Cont)(
+        concept (convertible_to_cont_impl_)(Rng, Cont),
+            constructible_from<range_value_t<Cont>, range_reference_t<Rng>> AND
+            constructible_from<
+                Cont,
+                range_cpp17_iterator_t<Rng>,
+                range_cpp17_iterator_t<Rng>>
         );
+        template<typename Rng, typename Cont>
+        CPP_concept convertible_to_cont = //
+            range_and_not_view<Cont> && //
+            move_constructible<Cont> && //
+            CPP_concept_ref(detail::convertible_to_cont_impl_, Rng, Cont);
 
-        CPP_def
-        (
-            template(typename Rng, typename Cont)
-            concept convertible_to_container,
-                defer::has_allocator_type<Cont> && // HACKHACK
-                defer::convertible_to_container_impl_<Rng, Cont>
+        template(typename Rng, typename Cont)(
+        concept (convertible_to_cont_cont_impl_)(Rng, Cont),
+            range_and_not_view<range_value_t<Cont>> AND
+            // Test that each element of the input range can be ranges::to<>
+            // to the output container.
+            invocable<
+                to_container::fn<meta::id<range_value_t<Cont>>>,
+                range_reference_t<Rng>> AND
+            constructible_from<
+                Cont,
+                to_container_iterator_t<Rng, Cont>,
+                to_container_iterator_t<Rng, Cont>>
         );
+        template<typename Rng, typename Cont>
+        CPP_concept convertible_to_cont_cont = //
+            range<Cont> && //
+            (!view_<Cont>) && //
+            move_constructible<Cont> && //
+            CPP_concept_ref(detail::convertible_to_cont_cont_impl_, Rng, Cont);
 
-        CPP_def
-        (
-            template(typename Rng, typename Cont)
-            concept convertible_to_container_container,
-                defer::has_allocator_type<Cont> && // HACKHACK
-                defer::convertible_to_container_container_impl_<Rng, Cont>
-        );
+        template<typename C, typename I, typename R>
+        CPP_concept to_container_reserve = //
+            reservable_with_assign<C, I> && //
+            sized_range<R>;
 
-        CPP_def
-        (
-            template(typename C, typename I, typename R)
-            concept to_container_reserve,
-                reservable_with_assign<C, I> &&
-                sized_range<R>
-        );
+        template<typename MetaFn, typename Rng>
+        using container_t = meta::invoke<MetaFn, Rng>;
         // clang-format on
 
-        template<typename ToContainer>
-        struct to_container::fn : pipeable_base
+        struct RANGES_STRUCT_WITH_ADL_BARRIER(to_container_closure_base)
+        {
+            // clang-format off
+            template(typename Rng, typename MetaFn, typename Fn)(
+                /// \pre
+                requires input_range<Rng> AND
+                    convertible_to_cont<Rng, container_t<MetaFn, Rng>>)
+            friend constexpr auto
+            operator|(Rng && rng, to_container::closure<MetaFn, Fn> fn)
+            {
+                return static_cast<Fn &&>(fn)(static_cast<Rng &&>(rng));
+            }
+
+            template(typename Rng, typename MetaFn, typename Fn)(
+                /// \pre
+                requires input_range<Rng> AND
+                    (!convertible_to_cont<Rng, container_t<MetaFn, Rng>>) AND
+                    convertible_to_cont_cont<Rng, container_t<MetaFn, Rng>>)
+            friend constexpr auto
+            operator|(Rng && rng, to_container::closure<MetaFn, Fn> fn)
+            {
+                return static_cast<Fn &&>(fn)(static_cast<Rng &&>(rng));
+            }
+
+            template<typename MetaFn, typename Fn, typename Pipeable>
+            friend constexpr auto operator|(to_container::closure<MetaFn, Fn> sh,
+                                            Pipeable pipe)
+                -> CPP_broken_friend_ret(
+                    to_container::closure<MetaFn, composed<Pipeable, Fn>>)(
+                    /// \pre
+                    requires is_pipeable_v<Pipeable>)
+            {
+                return to_container::closure<MetaFn, composed<Pipeable, Fn>>{
+                    compose(static_cast<Pipeable &&>(pipe), static_cast<Fn &&>(sh))};
+            }
+        };
+
+        template<typename MetaFn, typename Fn>
+        struct to_container::closure
+          : to_container_closure_base
+          , Fn
+        {
+            closure() = default;
+            constexpr explicit closure(Fn fn)
+              : Fn(static_cast<Fn &&>(fn))
+            {}
+        };
+
+        template<typename MetaFn>
+        struct to_container::fn
         {
         private:
             template<typename Cont, typename I, typename Rng>
@@ -265,33 +330,32 @@ namespace ranges
                 c.assign(I{ranges::begin(rng)}, I{ranges::end(rng)});
                 return c;
             }
-            template<typename Rng>
-            using container_t = meta::invoke<ToContainer, Rng>;
 
         public:
-            template<typename Rng>
-            auto operator()(Rng && rng) const -> CPP_ret(container_t<Rng>)(           //
-                requires input_range<Rng> &&                                          //
-                (bool(!defer::convertible_to_container_container<Rng,                 //
-                                                                 container_t<Rng>> && //
-                      defer::convertible_to_container<Rng, container_t<Rng>>)))
+            template(typename Rng)(
+                /// \pre
+                requires input_range<Rng> AND
+                    convertible_to_cont<Rng, container_t<MetaFn, Rng>>)
+            container_t<MetaFn, Rng> operator()(Rng && rng) const
             {
                 static_assert(!is_infinite<Rng>::value,
                               "Attempt to convert an infinite range to a container.");
-                using cont_t = container_t<Rng>;
+                using cont_t = container_t<MetaFn, Rng>;
                 using iter_t = range_cpp17_iterator_t<Rng>;
                 using use_reserve_t =
                     meta::bool_<(bool)to_container_reserve<cont_t, iter_t, Rng>>;
                 return impl<cont_t, iter_t>(static_cast<Rng &&>(rng), use_reserve_t{});
             }
-            template<typename Rng>
-            auto operator()(Rng && rng) const -> CPP_ret(container_t<Rng>)( //
-                requires input_range<Rng> &&                                //
-                    convertible_to_container_container<Rng, container_t<Rng>>)
+            template(typename Rng)(
+                /// \pre
+                requires input_range<Rng> AND
+                    (!convertible_to_cont<Rng, container_t<MetaFn, Rng>>) AND
+                    convertible_to_cont_cont<Rng, container_t<MetaFn, Rng>>)
+            container_t<MetaFn, Rng> operator()(Rng && rng) const
             {
                 static_assert(!is_infinite<Rng>::value,
                               "Attempt to convert an infinite range to a container.");
-                using cont_t = container_t<Rng>;
+                using cont_t = container_t<MetaFn, Rng>;
                 using iter_t = to_container_iterator<Rng, cont_t>;
                 using use_reserve_t =
                     meta::bool_<(bool)to_container_reserve<cont_t, iter_t, Rng>>;
@@ -299,8 +363,11 @@ namespace ranges
             }
         };
 
-        template<typename ToContainer>
-        using to_container_fn = to_container::fn<ToContainer>;
+        template<typename MetaFn, typename Fn>
+        using to_container_closure = to_container::closure<MetaFn, Fn>;
+
+        template<typename MetaFn>
+        using to_container_fn = to_container_closure<MetaFn, to_container::fn<MetaFn>>;
 
         template<template<typename...> class ContT>
         struct from_range
@@ -308,12 +375,13 @@ namespace ranges
 #if RANGES_CXX_DEDUCTION_GUIDES >= RANGES_CXX_DEDUCTION_GUIDES_17
             // Attempt to use a deduction guide first...
             template<typename Rng>
-            static auto from_rng_(int) -> decltype(ContT(range_cpp17_iterator_t<Rng>{},
-                                                         range_cpp17_iterator_t<Rng>{}));
+            static auto from_rng_(int) //
+                -> decltype(ContT(range_cpp17_iterator_t<Rng>{},
+                                  range_cpp17_iterator_t<Rng>{}));
             // No deduction guide. Fallback to instantiating with the
             // iterator's value type.
             template<typename Rng>
-            static auto from_rng_(long)
+            static auto from_rng_(long) //
                 -> meta::invoke<meta::quote<ContT>, range_value_t<Rng>>;
 
             template<typename Rng>
@@ -348,10 +416,11 @@ namespace ranges
         }
 
         /// \overload
-        template<template<typename...> class ContT, typename Rng>
-        auto to(Rng && rng) -> CPP_ret(ContT<range_value_t<Rng>>)( //
-            requires range<Rng> &&
-                detail::convertible_to_container<Rng, ContT<range_value_t<Rng>>>)
+        template(template<typename...> class ContT, typename Rng)(
+            /// \pre
+            requires range<Rng> AND
+                detail::convertible_to_cont<Rng, ContT<range_value_t<Rng>>>)
+        auto to(Rng && rng) -> ContT<range_value_t<Rng>>
         {
             return detail::to_container_fn<detail::from_range<ContT>>{}(
                 static_cast<Rng &&>(rng));
@@ -366,24 +435,27 @@ namespace ranges
         }
 
         /// \overload
-        template<typename Cont, typename Rng>
-        auto to(Rng && rng) -> CPP_ret(Cont)( //
-            requires range<Rng> && detail::convertible_to_container<Rng, Cont>)
+        template(typename Cont, typename Rng)(
+            /// \pre
+            requires range<Rng> AND detail::convertible_to_cont<Rng, Cont>)
+        auto to(Rng && rng) -> Cont
         {
             return detail::to_container_fn<meta::id<Cont>>{}(static_cast<Rng &&>(rng));
         }
 
         /// \cond
         // Slightly odd initializer_list overloads, undocumented for now.
-        template<template<typename...> class ContT, typename T>
-        auto to(std::initializer_list<T> il) -> CPP_ret(ContT<T>)( //
-            requires detail::convertible_to_container<std::initializer_list<T>, ContT<T>>)
+        template(template<typename...> class ContT, typename T)(
+            /// \pre
+            requires detail::convertible_to_cont<std::initializer_list<T>, ContT<T>>)
+        auto to(std::initializer_list<T> il) -> ContT<T>
         {
             return detail::to_container_fn<detail::from_range<ContT>>{}(il);
         }
-        template<typename Cont, typename T>
-        auto to(std::initializer_list<T> il) -> CPP_ret(Cont)( //
-            requires detail::convertible_to_container<std::initializer_list<T>, Cont>)
+        template(typename Cont, typename T)(
+            /// \pre
+            requires detail::convertible_to_cont<std::initializer_list<T>, Cont>)
+        auto to(std::initializer_list<T> il) -> Cont
         {
             return detail::to_container_fn<meta::id<Cont>>{}(il);
         }
@@ -402,48 +474,57 @@ namespace ranges
         // The old name "ranges::to_" is now deprecated:
         template<template<typename...> class ContT>
         RANGES_DEPRECATED("Please use ranges::to (no underscore) instead.")
-        auto to_(detail::to_container = {})
-            -> detail::to_container_fn<detail::from_range<ContT>>
+        detail::to_container_fn<detail::from_range<ContT>> to_(detail::to_container = {})
         {
             return {};
         }
-        template<template<typename...> class ContT, typename Rng>
+        template(template<typename...> class ContT, typename Rng)(
+            /// \pre
+            requires range<Rng> AND
+                detail::convertible_to_cont<Rng, ContT<range_value_t<Rng>>>)
         RANGES_DEPRECATED("Please use ranges::to (no underscore) instead.")
-        auto to_(Rng && rng) -> CPP_ret(ContT<range_value_t<Rng>>)( //
-            requires range<Rng> &&
-                detail::convertible_to_container<Rng, ContT<range_value_t<Rng>>>)
+        ContT<range_value_t<Rng>> to_(Rng && rng)
         {
             return static_cast<Rng &&>(rng) | ranges::to_<ContT>();
         }
-        template<template<typename...> class ContT, typename T>
+        template(template<typename...> class ContT, typename T)(
+            /// \pre
+            requires detail::convertible_to_cont<std::initializer_list<T>, ContT<T>>)
         RANGES_DEPRECATED("Please use ranges::to (no underscore) instead.")
-        auto to_(std::initializer_list<T> il) -> CPP_ret(ContT<T>)( //
-            requires detail::convertible_to_container<std::initializer_list<T>, ContT<T>>)
+        ContT<T> to_(std::initializer_list<T> il)
         {
             return il | ranges::to_<ContT>();
         }
         template<typename Cont>
         RANGES_DEPRECATED("Please use ranges::to (no underscore) instead.")
-        auto to_(detail::to_container = {}) -> detail::to_container_fn<meta::id<Cont>>
+        detail::to_container_fn<meta::id<Cont>> to_(detail::to_container = {})
         {
             return {};
         }
-        template<typename Cont, typename Rng>
+        template(typename Cont, typename Rng)(
+            /// \pre
+            requires range<Rng> AND detail::convertible_to_cont<Rng, Cont>)
         RANGES_DEPRECATED("Please use ranges::to (no underscore) instead.")
-        auto to_(Rng && rng) -> CPP_ret(Cont)( //
-            requires range<Rng> && detail::convertible_to_container<Rng, Cont>)
+        Cont to_(Rng && rng)
         {
             return static_cast<Rng &&>(rng) | ranges::to_<Cont>();
         }
-        template<typename Cont, typename T>
+        template(typename Cont, typename T)(
+            /// \pre
+            requires detail::convertible_to_cont<std::initializer_list<T>, Cont>)
         RANGES_DEPRECATED("Please use ranges::to (no underscore) instead.")
-        auto to_(std::initializer_list<T> list) -> CPP_ret(Cont)( //
-            requires detail::convertible_to_container<std::initializer_list<T>, Cont>)
+        Cont to_(std::initializer_list<T> list)
         {
             return list | ranges::to_<Cont>();
         }
     } // namespace _to_
     /// \endcond
+
+    template<typename MetaFn, typename Fn>
+    RANGES_INLINE_VAR constexpr bool
+        is_pipeable_v<detail::to_container_closure<MetaFn, Fn>> = true;
 } // namespace ranges
 
+#include <range/v3/detail/epilogue.hpp>
+
 #endif
