@@ -24,6 +24,8 @@
 #include <range/v3/utility/static_const.hpp>
 #include <range/v3/view/facade.hpp>
 
+#include <range/v3/detail/prologue.hpp>
+
 namespace ranges
 {
     /// \addtogroup group-views
@@ -107,10 +109,10 @@ namespace ranges
     {
         struct repeat_n_fn
         {
-            template<typename Val>
-            auto operator()(Val value, std::ptrdiff_t n) const
-                -> CPP_ret(repeat_n_view<Val>)( //
-                    requires copy_constructible<Val>)
+            template(typename Val)(
+                /// \pre
+                requires copy_constructible<Val>)
+            repeat_n_view<Val> operator()(Val value, std::ptrdiff_t n) const
             {
                 return repeat_n_view<Val>{std::move(value), n};
             }
@@ -123,6 +125,7 @@ namespace ranges
     /// @}
 } // namespace ranges
 
+#include <range/v3/detail/epilogue.hpp>
 #include <range/v3/detail/satisfy_boost_range.hpp>
 RANGES_SATISFY_BOOST_RANGE(::ranges::repeat_n_view)
 
